Ext.namespace('dSS', 'dSS.grid', 'dSS.data', 'dSS.config', 'dSA',
              'dSS11.config');

dSS.grid.ModulatorPanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {

        var dSIDrenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('dsid'));
            return String.format('<div qtip="{0}">{1}</div>',
                                  record.get('dsid'), shortId);
        };

        var modulatorColumns = [
        {
            id: 'name',
            header: dSS._("dSM name"),
            tooltip: dSS._("dSM name"),
            width: 80,
            sortable: true,
            dataIndex: 'name'
        },
        {
            id: 'dsid',
            header: dSS._("dSM dSID"),
            tooltip: dSS._("dSM dSID"),
            width: 72,
            sortable: true,
            dataIndex: 'dsid',
            renderer: dSIDrenderer
        },
        {
            id: 'numdevices',
            header: dSS._("Number of devices"),
            tooltip: dSS._("Number of devices"),
            sortable: true,
            dataIndex: 'numDevices'
        },
        {
            id: 'consumption',
            header: dSS._("Consumption (W)"),
            tooltip: dSS._("Consumption (W)"),
            width: 30,
            sortable: true,
            dataIndex: 'consumption'
        },
/*
        {
            id: 'measureDate',
            header: dSS._("measured"),
            tooltip: dSS._("measured"),
            width: 60,
            sortable: true,
            dataIndex: 'measureDate'
        },
*/
         {
            id: 'armSwVersion',
            header: dSS._("ARM software version"),
            tooltip: dSS._("ARM software version"),
            width: 30,
            sortable: true,
            dataIndex: 'armSwVersion'
        },
        {
            id: 'dspSwVersion',
            header: dSS._("DSP software version"),
            tooltip: dSS._("DSP software version"),
            width: 30,
            sortable: true,
            dataIndex: 'dspSwVersion'
        },
        {
            id: 'hwVersion',
            header: dSS._("HW version"),
            tooltip: dSS._("Hardware version"),
            width: 30,
            sortable: true,
            dataIndex: 'hwVersion'
        }
/*
        {
            id: 'apiVersion',
            header: dSS._("API version"),
            tooltip: dSS._("API version"),
            width: 20,
            sortable: true,
            dataIndex: 'apiVersion'
        }
*/
        ];

        var rowSelect = new Ext.grid.RowSelectionModel({
            singleSelect: false
        });

        this.heightConfigured = false;

        Ext.apply(this, {
            store: dSS.data.Loader.getModulatorStore(),
            columns: modulatorColumns,
            enableDragDrop: false,
            stripeRows: true,
            title: dSS._("Meters"),
            selModel: rowSelect,
            viewConfig: {
                forceFit: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.ModulatorPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
            if (this.heightConfigured === false) {
                this.heightConfigured = true;
                this.setHeight(this.findParentByType('dsmoverviewpanel').getInnerHeight() / 2);
            }

        });

        this.on('viewready', function() {
            this.setInitialSelection();
        });

        dSS.MessageBus.on('structureloaded', function() {
            this.setInitialSelection();
        }, this);

        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();

            if (!sel.hasSelection()) {
                sel.selectRow(rowIndex, false);
            }

            var showRemoveMenuItem = false;

            var records = sel.getSelections();

            var i;
            for (i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                    break;
                }
            }

            var menuItem;

            menuItem = new Ext.menu.Item({
                iconCls: 'blinkdSMIcon',
                text: dSS._("Blink dSM LED"),
                handler: this.blinkdSM.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                iconCls: 'transmissionQuality',
                text: dSS._("Test transmission quality"),
                handler: this.testTransmissionQuality.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                iconCls: 'checkDevicePresenceIcon',
                text: dSS._("Check device presence"),
                handler: this.checkDevicePresence.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                iconCls: 'reregisterDevicesIcon',
                text: dSS._("Reregister all devices"),
                handler: this.restartRegistration.createDelegate(this, rowIndex,
                                                                 true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                text: dSS._("Rename dSM"),
                iconCls: 'editIcon',
                handler: this.renameMeter.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                iconCls: 'viewCommunicationLogIcon',
                text: dSS._("View communication log"),
                handler: this.viewCommLog.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                text: dSS._("Remove dS-Meter"),
                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeMeter.createDelegate(this, rowIndex, true),
                disabled: !showRemoveMenuItem
            });
            this.contextMenu.add(menuItem);

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);

        rowSelect.on('rowselect', function(selModel, rowIndex, dataRecord) {
            this.filterDeviceStore(dataRecord);
        },
        this);

        dSS.MessageBus.on('deviceattachementchanged', function(dSMRecord) {
            this.filterDeviceStore(dSMRecord);
        },
        this);
    },
    filterDeviceStore: function(dSMRecord) {
        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        store.filterBy(function(record) {
            if (dSS.util.is2WaySlave(record.get('buttonInputMode'))) {
                return false;
            }
            if (record.get('meterDSID') === dSMRecord.get('dsid')) {
                return true;
            }
            return false;
        });
    },
    buildTopToolbar: function() {
        return [
        {
            iconCls: 'blinkdSMIcon',
            scope: this,
            handler: this.blinkdSM,
            tooltip: dSS._("Blink dSM LED")
        },
        '-',
        {
            iconCls: 'transmissionQuality',
            scope: this,
            handler: this.testTransmissionQuality,
            tooltip: dSS._("Test transmission quality")
        },
        {
            iconCls: 'checkDevicePresenceIcon',
            scope: this,
            handler: this.checkDevicePresence,
            tooltip: dSS._("Check device presence")
        },
        {
            iconCls: 'reregisterDevicesIcon',
            scope: this,
            handler: this.restartRegistration,
            tooltip: dSS._("Reregister all devices")
        },
        '-',
        {
            iconCls: 'editIcon',
            scope: this,
            handler: this.renameMeter,
            tooltip: dSS._("Rename dSM")
        },
        '->',
        {
            iconCls: 'viewCommunicationLogIcon',
            scope: this,
            handler: this.viewCommLog,
            tooltip: dSS._("View communication log")
        },
        '-',
        {
            iconCls: 'removeDeviceToolbarIcon',
            handler: this.removeMeter,
            tooltip: dSS._("Remove dS-Meter"),
            scope: this
        }];
    },

    renameMeter: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(dSS._("Rename meter"), dSS._("Name for") + ' ' + dSS.util.shortDSID(record.get('dsid')) + ': ', function(btn, text) {
            if ((btn === 'ok') && (text !== record.get('name'))) {
                Ext.Ajax.request({
                    url: '/json/circuit/setName',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        id: record.get('dsid'),
                        newName: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('name', text);
                                record.commit();

                                var devStore = dSS.data.Loader.getDeviceStore();
                                devStore.each(
                                    function(devRecord) {
                                        if (devRecord.get('meterDSID') ===
                                                record.get('dsid')) {
                                            devRecord.set('modulatorName',
                                                           text);
                                            devRecord.commit();
                                        }
                                    }
                                );
                                dSS.MessageBus.fireEvent('statusmessage',
                                    dSS._("digitalSTROM Meter") + ' ' +
                                    dSS.util.shortDSID(record.get('dsid')) +
                                    ' ' + dSS._("was renamed"));
                            } else {
                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename meter"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename meter"));
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename meter"));
                    }
                });
            }
        },
        this, false, record.get('name'));
    },
    setInitialSelection: function() {
        if (!this.isVisible()) {
            return;
        }

        if (this.getStore().getCount() === 0) {
            this.getStore().clearFilter();
            return;
        }

        var record;
        var index;

        if (this.getSelectionModel().hasSelection()) {
            index = this.getStore().indexOf(this.getSelectionModel().getSelected());
            record = this.getSelectionModel().getSelected();
        } else {
            this.getView().focusRow(0);
            this.getSelectionModel().selectFirstRow();
            record = this.getStore().getAt(0);
            index = 0;
        }

        this.getSelectionModel().fireEvent('rowselect', this.getSelectionModel(), index, record);
    },
    rescanCircuit: function(item, event, rowIndex) {
        var record = this.getStore().getAt(rowIndex);
        Ext.Ajax.request({
            url: '/json/circuit/rescan',
            disableCaching: true,
            method: "GET",
            params: {
                id: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok === false) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Rescan failed"));
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Rescan failed") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Rescan failed"));
            }
        });
    },
    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 120,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    removeMeter: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no meter selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No meter selected, please select a meter from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteMeter(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Present meters can not be removed!"));
            }
        }
    },
    deleteMeter: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.el.unmask();
            return;
        }

        var record = records.shift();

        this.el.mask(dSS._("Deleting meter") + ": " + record.get("dsid"));
        Ext.Ajax.request({
            url: '/json/apartment/removeMeter',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("digitalSTROM Meter") + ' ' +
                            dSS.util.shortDSID(record.get('dsid')) +
                            ' ' + dSS._("was removed"));

                        this.deleteMeter(records);
                    } else {
                        this.el.unmask();
                        Ext.MessageBox.alert(dSS._("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    this.el.unmask();
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove meter command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                this.el.unmask();
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove meter command to dSS"));
            }
        });
    },
    removeInactiveMeters: function() {
        Ext.Ajax.request({
            url: '/json/apartment/removeInactiveMeters',
            disableCaching: true,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        dSS.data.Loader.load();
                    } else {
                        Ext.MessageBox.alert(dSS._("Could not remove inactive meters"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove inactive meters command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove inactive meters command to dSS"));
            }
        });
    },
    getMeter: function(rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that
            // there is no selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No meter selected, please select a meter from " +
                          "the list!"));
                return null;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        // this should never happen
        if (record === undefined) {
            Ext.Msg.alert(dSS._("Error"), dSS._("dSM not found"));
            return null;
        }

        return record;
    },
    getMeter4dSA: function(rowIndex, ignoreNotPresent, notPresentMsg) {
        if (dSA.module !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature requires presence of the " +
                                "digitalSTROM Assistant, the setup is only " +
                                "available on the dSS11 hardware"));
        } else {
            var record = this.getMeter(rowIndex);
            if (record !== null) {
                if ((ignoreNotPresent === false) &&
                    (record.get('isPresent') === false)) {
                    Ext.Msg.alert(dSS._("Error"), notPresentMsg);
                    return null;
                }
            }

            return record;
        }

        return null;
    },
    restartRegistration: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false,
                                       dSS._("Can not restart " +
                                       "registration, dSM is not present!"));
        if (record !== null) {
            dSA.Reactor.fireEvent('reregisterdevices', record.get('dsid'));
        }
    },
    blinkdSM: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false,
                                       dSS._("Can blink, dSM is not present!"));
        if (record !== null) {
            dSA.Reactor.fireEvent('blinkdsmled', record.get('dsid'));
        }
    },
    testTransmissionQuality: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._("Can not start " +
                             "transmission quality test, dSM is not present!"));
        if (record !== null) {
            dSA.Reactor.fireEvent('checktransmissionquality',
                                  record.get('dsid'), record.get('name'),
                                  this.findParentByType(Ext.Viewport).el);
        }
    },
    checkDevicePresence: function(item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._("Can not start " +
                             "device presence test, dSM is not present!"));
        if (record !== null) {
            dSA.Reactor.fireEvent('checkdevicepresence',
                                  record.get('dsid'), record.get('name'),
                                  this.findParentByType(Ext.Viewport).el);
        }
    },
    // only the dSS11 UI is capable of showing logs, so we will send a message
    // with the appropriate dsid and the dSS11 websetup will take care of the
    // rest
    viewCommLog: function(item, event, rowIndex) {
        if (dSS11.config.dSSIntegration !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature is only available in the " +
                                "setup on dSS11 hardware"));
        } else {
            var record = this.getMeter(rowIndex);
            if (record !== null) {
                dSS.MessageBus.fireEvent('showlog', record.get('dsid'));
            }
        }
    }
});

Ext.reg('dssmodulatorpanel', dSS.grid.ModulatorPanel);

